package cn.com.ctop.crawler.modules.core.util;

import cn.com.ctop.crawler.modules.appium.entity.AppiumTaskItem;
import com.google.gson.Gson;
import io.appium.java_client.TouchAction;
import io.appium.java_client.android.AndroidDriver;
import io.appium.java_client.touch.WaitOptions;
import io.appium.java_client.touch.offset.PointOption;
import org.openqa.selenium.Point;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.remote.DesiredCapabilities;

import java.net.URL;
import java.time.Duration;
import java.util.List;

public class AppiumUtil {
    private Boolean isLoop = true;
    private Duration duration = Duration.ofSeconds(1);

    public void swipeToUp(AndroidDriver driver) {
        int width = driver.manage().window().getSize().width;
        int height = driver.manage().window().getSize().height;
        TouchAction action1 = new TouchAction(driver).press(PointOption.point(width / 2, height * 3 / 4)).waitAction(WaitOptions.waitOptions(duration))
                .moveTo(PointOption.point(width / 2, height / 4)).release();
        action1.perform();
    }

    public void swipeToDown(AndroidDriver driver) {
        int width = driver.manage().window().getSize().width;
        int height = driver.manage().window().getSize().height;
        TouchAction action2 = new TouchAction(driver).press(PointOption.point(width / 2, height / 4)).waitAction(WaitOptions.waitOptions(duration))
                .moveTo(PointOption.point(width / 2, height * 3 / 4)).release();
        action2.perform();
    }

    public void swipeToLeft(AndroidDriver driver) {
        int width = driver.manage().window().getSize().width;
        int height = driver.manage().window().getSize().height;
        TouchAction action3 = new TouchAction(driver).press(PointOption.point(width * 3 / 4, height / 2)).waitAction(WaitOptions.waitOptions(duration))
                .moveTo(PointOption.point(width / 4, height / 2)).release();
        action3.perform();
    }

    public void tapPoint(AndroidDriver driver, Integer x, Integer y) {
        TouchAction action = new TouchAction(driver).tap(PointOption.point(x, y)).release();
        action.perform();
    }

    public void swipeToRight(AndroidDriver driver) {
        int width = driver.manage().window().getSize().width;
        int height = driver.manage().window().getSize().height;
        TouchAction action4 = new TouchAction(driver).press(PointOption.point(width / 4, height / 2)).waitAction(WaitOptions.waitOptions(duration))
                .moveTo(PointOption.point(width * 3 / 4, height / 2)).release();
        action4.perform();
    }

    public AndroidDriver initAndroidDriver(String serverUrl, String deviceName, String appPackage, String appActivity) throws Exception {
        return new AndroidDriver(new URL(serverUrl), initCapabilities(deviceName, appPackage, appActivity));
    }

    public DesiredCapabilities initCapabilities(String deviceName, String appPackage, String appActivity) {
        DesiredCapabilities desiredCapabilities = new DesiredCapabilities();
        desiredCapabilities.setCapability("deviceName", deviceName);
        desiredCapabilities.setCapability("platformName", "Android");
        desiredCapabilities.setCapability("appPackage", appPackage);
        desiredCapabilities.setCapability("noReset", true);
        desiredCapabilities.setCapability("unicodeKeyboard", true);
        desiredCapabilities.setCapability("resetKeyboard", true);
        desiredCapabilities.setCapability("appActivity", appActivity);
        return desiredCapabilities;
    }

    public WebElement getElementById(AndroidDriver androidDriver, String id) {
        WebElement element = null;
        List<WebElement> elements = androidDriver.findElementsById(id);
        if (elements != null && elements.size() > 0) {
            element = elements.get(0);
        }
        return element;
    }

    public WebElement getElementByClass(AndroidDriver androidDriver, String className) {
        WebElement element = null;
        List<WebElement> elements = androidDriver.findElementsByClassName(className);
        if (elements != null && elements.size() > 0) {
            element = elements.get(0);
        }
        return element;
    }

    public WebElement getElementByXpath(AndroidDriver androidDriver, String xpath) {
        WebElement element = null;
        List<WebElement> elements = androidDriver.findElementsByXPath(xpath);
        if (elements != null && elements.size() > 0) {
            element = elements.get(0);
        }
        return element;
    }

    public void runTask(AndroidDriver androidDriver, AppiumTaskItem appiumTaskItem, WebElement loopElement) throws Exception {
        if (appiumTaskItem != null) {
            System.out.println(new Gson().toJson(appiumTaskItem));
            if (appiumTaskItem.getWaitTime() != null && appiumTaskItem.getWaitTime() > 0) {
                Thread.sleep(appiumTaskItem.getWaitTime());
            }
            if (appiumTaskItem.getLoopType() != null && appiumTaskItem.getLoopType().equals("noloop")) {
                if (appiumTaskItem.getFindType() != null) {
                    WebElement element = null;
                    if (appiumTaskItem.getFindType().equals("id")) {
                        element = getElementById(androidDriver, appiumTaskItem.getFindKey());
                    } else if (appiumTaskItem.getFindType().equals("class")) {
                        element = getElementByClass(androidDriver, appiumTaskItem.getFindKey());
                    } else if (appiumTaskItem.getFindType().equals("xpath")) {
                        element = getElementByXpath(androidDriver, appiumTaskItem.getFindKey());
                    } else if (appiumTaskItem.getFindType().equals("loop")) {
                        element = loopElement;
                    }
                    if (element != null) {
                        if (appiumTaskItem.getTextEqualType() != null) {
                            if (appiumTaskItem.getTextEqualType().equals("eq")) {
                                if (!element.getText().equals(appiumTaskItem.getTextEqualKey())) {
                                    return;
                                }
                            } else if (appiumTaskItem.getTextEqualType().equals("neq")) {
                                if (element.getText() == null || element.getText().equals(appiumTaskItem.getTextEqualKey())) {
                                    return;
                                }
                            }
                        }
                        if (appiumTaskItem.getClickType() != null) {
                            if (appiumTaskItem.getClickType().equals("element")) {
                                element.click();
                            } else if (appiumTaskItem.getClickType().equals("point")) {
                                Point point = element.getLocation();
                                tapPoint(androidDriver, point.getX() + appiumTaskItem.getOffsiteX(), point.getY() + appiumTaskItem.getOffsiteY());
                            }
                        }
                        List<AppiumTaskItem> subTask = appiumTaskItem.getSubList();
                        if (subTask != null && subTask.size() > 0) {
                            for (AppiumTaskItem task : subTask) {
                                runTask(androidDriver, task, element);
                            }
                        }
                    } else {
                        if (appiumTaskItem.getIsMust() != null && appiumTaskItem.getIsMust() == 1) {
                            if (appiumTaskItem.getLoopRate() != null && appiumTaskItem.getLoopRate() > 0) {
                                Thread.sleep(appiumTaskItem.getLoopRate());
                            }
                            runTask(androidDriver, appiumTaskItem, element);
                            return;
                        }
                        if (appiumTaskItem.getClickType() != null && appiumTaskItem.getClickType().equals("point")) {
                            tapPoint(androidDriver, appiumTaskItem.getOffsiteX(), appiumTaskItem.getOffsiteY());
                        }
                    }
                }
                if (appiumTaskItem.getSwapType() != null) {
                    if (appiumTaskItem.getSwapType().equals("up")) {
                        this.swipeToUp(androidDriver);
                    } else if (appiumTaskItem.getSwapType().equals("down")) {
                        this.swipeToDown(androidDriver);
                    } else if (appiumTaskItem.getSwapType().equals("left")) {
                        this.swipeToLeft(androidDriver);
                    } else if (appiumTaskItem.getSwapType().equals("right")) {
                        this.swipeToRight(androidDriver);
                    }
                }
            } else if (appiumTaskItem.getLoopType() != null && appiumTaskItem.getLoopType().equals("loop")) {
                while (isLoop) {
                    System.out.println("loop:" + isLoop);
                    if (appiumTaskItem.getFindType() != null) {
                        List<WebElement> list = null;
                        if (appiumTaskItem.getFindType().equals("id")) {
                            list = androidDriver.findElementsById(appiumTaskItem.getFindKey());
                        } else if (appiumTaskItem.getFindType().equals("class")) {
                            list = androidDriver.findElementsByClassName(appiumTaskItem.getFindKey());
                        } else if (appiumTaskItem.getFindType().equals("xpath")) {
                            list = androidDriver.findElementsByXPath(appiumTaskItem.getFindKey());
                        }
                        if (list != null && list.size() > 0) {
                            for (WebElement element : list) {
                                if (!isLoop) {
                                    break;
                                }
                                if (appiumTaskItem.getLoopRate() != null && appiumTaskItem.getLoopRate() > 0) {
                                    Thread.sleep(appiumTaskItem.getLoopRate());
                                }
                                List<AppiumTaskItem> subTask = appiumTaskItem.getSubList();
                                if (subTask != null && subTask.size() > 0) {
                                    for (AppiumTaskItem task : subTask) {
                                        runTask(androidDriver, task, element);
                                    }
                                }
                            }
                        }
                    }
                    if (appiumTaskItem.getSwapType() != null) {
                        if (appiumTaskItem.getSwapType().equals("up")) {
                            this.swipeToUp(androidDriver);
                        } else if (appiumTaskItem.getSwapType().equals("down")) {
                            this.swipeToDown(androidDriver);
                        } else if (appiumTaskItem.getSwapType().equals("left")) {
                            this.swipeToLeft(androidDriver);
                        } else if (appiumTaskItem.getSwapType().equals("right")) {
                            this.swipeToRight(androidDriver);
                        }
                    }
                }

            }
            if (appiumTaskItem.getIsClose() != null && appiumTaskItem.getIsClose() == 1) {
                this.isLoop = false;
            }
        }
    }
}
